// COpenGLView.cpp : implementation of the COpenGLView class
//

#include "stdafx.h"
#include "OpenGL View Class.h"

#include "OpenGL View ClassDoc.h"
#include "COpenGLView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



const char* const COpenGLView::_ErrorStrings[]= {
				{"No Error"},					// 0
				{"Unable to get a DC"},			// 1
				{"ChoosePixelFormat failed"},	// 2
				{"SelectPixelFormat failed"},	// 3
				{"wglCreateContext failed"},	// 4
				{"wglMakeCurrent failed"},		// 5
				{"wglDeleteContext failed"},	// 6
				{"SwapBuffers failed"},			// 7

		};

/////////////////////////////////////////////////////////////////////////////
// COpenGLView

IMPLEMENT_DYNCREATE(COpenGLView, CView)

BEGIN_MESSAGE_MAP(COpenGLView, CView)
	//{{AFX_MSG_MAP(COpenGLView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_SURFACE, OnSurface)
	ON_UPDATE_COMMAND_UI(ID_SURFACE, OnUpdateSurface)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenGLView construction/destruction

COpenGLView::COpenGLView() :
	 m_hRC(0), m_pDC(0), m_ErrorString(_ErrorStrings[0])

{
	// TODO: add construction code here
	starttime = clock();
	bSurface=false;

}


COpenGLView::~COpenGLView()
{
}

BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class

	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. 
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  	
	return CView::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// COpenGLView drawing


void COpenGLView::OnDraw(CDC* pDC)
{
	COpenGLViewClassDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here

	::glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	PreRenderScene();


	::glPushMatrix();
	RenderScene();
	::glPopMatrix();

	::glFinish();

	if ( FALSE == ::SwapBuffers( m_pDC->GetSafeHdc() ) )
		{
		SetError(7);
		}
	
	InvalidateRect(0,FALSE);
	GetParent()->PostMessage(WM_PAINT);

}


/////////////////////////////////////////////////////////////////////////////
// COpenGLView diagnostics

#ifdef _DEBUG
void COpenGLView::AssertValid() const
{
	CView::AssertValid();
}

void COpenGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

COpenGLViewClassDoc* COpenGLView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(COpenGLViewClassDoc)));
	return (COpenGLViewClassDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COpenGLView message handlers

int COpenGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here

	InitializeOpenGL();

	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// GL helper functions

void COpenGLView::SetError( int e )
{
	// if there was no previous error,
	// then save this one
	if ( _ErrorStrings[0] == m_ErrorString ) 
		{
		m_ErrorString = _ErrorStrings[e];
		}
}


BOOL COpenGLView::InitializeOpenGL()
{
	// Can we put this in the constructor?
    m_pDC = new CClientDC(this);

    if ( NULL == m_pDC ) // failure to get DC
		{
		SetError(1);
		return FALSE;
		}

	if (!SetupPixelFormat())
		{
        return FALSE;
		}

    //n = ::GetPixelFormat(m_pDC->GetSafeHdc());
    //::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

  //  CreateRGBPalette();

    if ( 0 == (m_hRC = ::wglCreateContext( m_pDC->GetSafeHdc() ) ) )
		{
		SetError(4);
		return FALSE;
		}

    if ( FALSE == ::wglMakeCurrent( m_pDC->GetSafeHdc(), m_hRC ) )
		{
		SetError(5);
		return FALSE;
		}	

	// specify black as clear color
    ::glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
	// specify the back of the buffer as clear depth
    ::glClearDepth( 1.0f );
	// enable depth testing
    ::glEnable( GL_DEPTH_TEST );

	return TRUE;
}


BOOL COpenGLView::SetupPixelFormat()
{
  static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
          PFD_SUPPORT_OPENGL |          // support OpenGL
          PFD_DOUBLEBUFFER,             // double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        0,                              // no alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
//        32,                             // 32-bit z-buffer
		16,	// NOTE: better performance with 16-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };
    int pixelformat;

    if ( 0 == (pixelformat = ::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) )
	    {
		SetError(2);
        return FALSE;
		}

    if ( FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) )
	    {
       	SetError(3);
        return FALSE;
		}

    return TRUE;
}

void COpenGLView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here

    if ( FALSE == ::wglDeleteContext( m_hRC ) )
		{
		SetError(6);
 		}

    if ( m_pDC )
		{
        delete m_pDC;
		}
}


BOOL COpenGLView::OnEraseBkgnd( CDC* pDC ) 
{
	// TODO: Add your message handler code here and/or call default
	
	//	return CView::OnEraseBkgnd(pDC);
	return TRUE; // tell Windows not to erase the background
}

void COpenGLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	GLdouble aspect_ratio; // width/height ratio
	
	if ( 0 >= cx || 0 >= cy )
		{
		return;
		}


	SetupViewport( cx, cy );

	// select the projection matrix and clear it
    ::glMatrixMode( GL_PROJECTION );
    ::glLoadIdentity();

	// compute the aspect ratio
	// this will keep all dimension scales equal
	aspect_ratio = (GLdouble)cx/(GLdouble)cy;

	// select the viewing volumn
	SetupViewingFrustum( aspect_ratio );
	
	// switch back to the modelview matrix
    ::glMatrixMode( GL_MODELVIEW );
    ::glLoadIdentity();

	// now perform any viewing transformations
	SetupViewingTransform();
}

 /////////////////////////////////////////////////////////////////////////////
// COpenGLView helper functions

BOOL COpenGLView::SetupViewport( int cx, int cy )
{
	// select the full client area
    ::glViewport(0, 0, cx, cy);

	return TRUE;
}

BOOL COpenGLView::SetupViewingFrustum( GLdouble aspect_ratio )
{
	// select a default viewing volumn
    ::gluPerspective( 40.0f, aspect_ratio, .1f, 20.0f );
	return TRUE;
}


BOOL COpenGLView::SetupViewingTransform()
{
	// select a default viewing transformation
	// of a 20 degree rotation about the X axis
	// then a -5 unit transformation along Z
	::glTranslatef( 0.0f, 0.0f, -5.0f );
	::glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );
    return TRUE;
}


BOOL COpenGLView::RenderScene()
{

	//::glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE );
	//::glLightf(GL_LIGHT0,GL_CONSTANT_ATTENUATION ,1.0);
	//::glLightfv(GL_LIGHT0,GL_AMBIENT ,white);
	//::glEnable(GL_NORMALIZE);

	glEnable(GL_LIGHTING);
	GLfloat local_amb[]={0.5f,0.8f,0.5f,1.0f};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,local_amb);
	glShadeModel(GL_FLAT);



	GLfloat pos[]={lightx,lighty,lightz};
	
	GLfloat white[]={1.0f,1.0f,1.0f,1.0f};

	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0,GL_SPECULAR ,white);
	glLightfv(GL_LIGHT0,GL_DIFFUSE ,white);
	glLightfv(GL_LIGHT0,GL_POSITION ,pos);

	GLfloat pos1[]={-3.0,0.0};
	GLfloat red[]={1.0f,0.0f,0.0f,1.0f};

//	::glEnable(GL_LIGHT1);
	::glLightfv(GL_LIGHT1,GL_SPECULAR ,red);
	::glLightfv(GL_LIGHT1,GL_DIFFUSE ,red);
	::glLightfv(GL_LIGHT1,GL_POSITION ,pos1);


	GLfloat mat_shine[]={128.0f};
	GLfloat mat_spec[]={0.60f,0.60f,1.0f,1.0f};
	GLfloat mat_diff[]={0.2f,0.5f,0.9f,1.0f};
	GLfloat mat_amb[]={0.8f,0.6f,0.1f,1.0f};


	glMaterialfv(GL_FRONT,GL_AMBIENT,mat_amb);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diff);
	glMaterialfv(GL_FRONT,GL_SHININESS,mat_shine);
	glMaterialfv(GL_FRONT,GL_SPECULAR,mat_spec);



	static int a=0;
	a++;
	glPushMatrix();
	glTranslatef(lightx,lighty,lightz);
	auxWireSphere(0.3);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(-1.5,0,0);
	glRotatef(a,1,0,0);
	auxSolidOctahedron(0.3);
	glPopMatrix();

	glRotatef(a,0,1,0);

	auxSolidSphere(1.0);


    return TRUE;
}	

#define TWO_PI 2*3.141
#define PI 3.141



void COpenGLView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	lightx=(float)point.x/200.0f-2;
	if(nFlags&MK_LBUTTON)
		lightz=2-(float)point.y/200.0f;
	else
		lighty=2-(float)point.y/200.0f;
	
	CView::OnMouseMove(nFlags, point);
}


GLfloat* COpenGLView::normal(GLfloat a[3],GLfloat b[3],GLfloat c[3],GLfloat r[3]){
	GLfloat x[3],y[3],z[3];

	x[0]=b[0]-a[0];
	x[1]=b[1]-a[1];
	x[2]=b[2]-a[2];

	y[0]=b[0]-c[0];
	y[1]=b[1]-c[1];
	y[2]=b[2]-c[2];

	r[0]=(x[1]*y[2]-y[1]*x[2]);
	r[1]=-(x[0]*y[2]-y[0]*x[2]);
	r[2]=(x[0]*y[1]-y[0]*x[1]);

	GLdouble d= r[0]*r[0]+r[1]*r[1]+r[2]*r[2];

	if(d<(GLdouble)0.00000001)
		d=100000000.0;
	else
		d=1.0/sqrt(d);

	r[0]*=d;
	r[1]*=d;
	r[2]*=d;
	
	//r[0]=a[0];
	//r[1]=a[1];
	//r[2]=a[2];
	return r;
}

void COpenGLView::OnSurface() 
{
	// TODO: Add your command handler code here
	bSurface=!bSurface;

	
}

void COpenGLView::OnUpdateSurface(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(bSurface);
	
}
